@@ -23,14 +23,8 @@
 #ifndef GS2_HELPER_H
 # define GS2_HELPER_H
 
-/* Get GSS-API functions. */
-# ifdef HAVE_LIBGSS
-#  include <gss.h>
-# elif HAVE_GSSAPI_H
-#  include <gssapi.h>
-# elif HAVE_GSSAPI_GSSAPI_H
-#  include <gssapi/gssapi.h>
-# endif
+/* For GSS-API types. */
+# include "gss-extra.h"
 
 /* Get gsasl functions and types. */
 # include <gsasl.h>
